package com.fjhx.dingding.service.impl;

import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.extra.pinyin.PinyinUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.dynamic.datasource.toolkit.DynamicDataSourceContextHolder;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.dingtalk.api.response.OapiV2DepartmentListsubResponse;
import com.fjhx.common.constant.SourceConstant;
import com.fjhx.dingding.entity.suite.po.SuiteTicketInfo;
import com.fjhx.dingding.service.DingService;
import com.fjhx.dingding.service.suite.SuiteTicketInfoService;
import com.fjhx.dingding.utils.DingUtil;
import com.fjhx.tenant.entity.tenant.po.TenantInfo;
import com.fjhx.tenant.service.tenant.impl.TenantInfoServiceImpl;
import com.ruoyi.common.constant.StatusConstant;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.SecurityUtils;
import com.ruoyi.system.service.ISysDeptService;
import com.ruoyi.system.service.ISysUserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
public class DingServiceImpl implements DingService {

    @Autowired
    private SuiteTicketInfoService suiteTicketInfoService;

    @Autowired
    private TenantInfoServiceImpl tenantInfoService;

    @Autowired
    private PlatformTransactionManager platformTransactionManager;

    @Autowired
    private TransactionDefinition transactionDefinition;

    @Autowired
    private ISysDeptService sysDeptService;

    @Autowired
    private ISysUserService sysUserService;

    /**
     * https://open.dingtalk.com/document/isvapp/data-formats
     */
    @Override
    public void saveCallback(JSONObject callBackContent) {

        JSONObject bizData = callBackContent.getJSONObject("bizData");

    }

    @Override
    public void pushHigh(JSONObject callBackContent) {
        JSONArray bizData = callBackContent.getJSONArray("bizData");

        for (JSONObject jsonObject : bizData.toJavaList(JSONObject.class)) {

            switch (jsonObject.getInteger("biz_type")) {

                case 2:
                    saveSuiteTicketInfo(jsonObject);
                    break;

                case 4:
                    corpAuthorization(jsonObject);
                    break;

            }

        }

    }

    public static void main(String[] args) {
        System.out.println("{\"auth_user_info\":{\"userId\":\"manager7560\"},\"eventId\":\"b542e727bad741d6b16de878676c363e\",\"auth_corp_info\":{\"corp_type\":0,\"corpid\":\"ding3f327504d104ac1cf2c783f7214b6d69\",\"auth_level\":0,\"corp_city\":\"福州\",\"auth_channel\":\"4\",\"industry\":\"计算机软件\",\"full_corp_name\":\"hf测试\",\"corp_name\":\"hf测试\",\"corp_province\":\"福建\",\"is_ecological_corp\":false,\"invite_url\":\"https://wx-in-i.dingtalk.com/invite-page/weixin.html?bizSource=____source____&corpId=ding3f327504d104ac1cf2c783f7214b6d69&inviteCode=i8yWYDEytZJIkIa\",\"auth_channel_type\":\"\",\"corp_type_v2\":0,\"invite_code\":\"\",\"is_authenticated\":false,\"license_code\":\"\",\"corp_logo_url\":\"\"},\"syncAction\":\"org_suite_auth\",\"auth_scope\":{\"errcode\":0,\"condition_field\":[],\"auth_user_field\":[\"jobnumber\",\"isLeader\",\"name\",\"isAdmin\",\"avatar\",\"position\",\"department\",\"userid\",\"deviceId\",\"isHide\"],\"auth_org_scopes\":{\"authed_user\":[],\"authed_dept\":[1]},\"errmsg\":\"ok\"},\"auth_info\":{\"agent\":[{\"agentid\":2581273014,\"agent_name\":\"byteSailing_test\",\"logo_url\":\"https://static-legacy.dingtalk.com/media/lALPDfJ6cdfiyqfNAZDNAZA_400_400.png\",\"appid\":131175,\"use_app_role\":false,\"admin_list\":[\"manager7560\"]}]},\"permanent_code\":\"caY2fuNclujXwRXlDOuctE-4HlOY6hWfgBTnMDbOaCPGQ-yzWQBxE6-axKq9XSJy\",\"syncSeq\":\"8F4DD3DFE04AA32FF1E6B2AE55\"}");
    }

    /**
     * 企业授权
     * <p>
     * https://open.dingtalk.com/document/isvapp/obtains-the-authorization-information-after-the-enterprise-activates-the-application
     */
    private void corpAuthorization(JSONObject bizData) {

        String itemBizDataStr = bizData.getString("biz_data");
        JSONObject jsonObject = JSON.parseObject(itemBizDataStr);

        String syncAction = jsonObject.getString("syncAction");

        switch (syncAction) {

            // 企业授权
            case "org_suite_auth":

                orgSuiteAuth(jsonObject);
                break;

            // 企业变更授权范围
            case "org_suite_change":

                break;

            // 表示企业解除授权
            case "org_suite_relieve":

                break;
        }

    }

    /**
     * 保存 suite_ticket
     */
    private void saveSuiteTicketInfo(JSONObject bizData) {
        String corpId = bizData.getString("corp_id");
        String suiteTicket = bizData.getJSONObject("biz_data").getString("suiteTicket");
        boolean update = suiteTicketInfoService.update(q -> q
                .eq(SuiteTicketInfo::getCorpId, corpId)
                .set(SuiteTicketInfo::getSuiteTicket, suiteTicket)
                .set(SuiteTicketInfo::getUpdateTime, new Date()));

        if (!update) {
            SuiteTicketInfo suiteTicketInfo = new SuiteTicketInfo();
            suiteTicketInfo.setCorpId(corpId);
            suiteTicketInfo.setSuiteTicket(suiteTicket);
            suiteTicketInfoService.save(suiteTicketInfo);
        }

    }

    /**
     * 企业授权
     */
    private void orgSuiteAuth(JSONObject bizData) {
        DynamicDataSourceContextHolder.push(SourceConstant.BASE);

        JSONObject authCorpInfo = bizData.getJSONObject("auth_corp_info");

        // 企业id
        String corpid = authCorpInfo.getString("corpid");
        // 企业名称
        String corpName = authCorpInfo.getString("corp_name");

        // 企业id存在则跳过
        if (tenantInfoService.getOne(q -> q.eq(TenantInfo::getCorpid, corpid)) != null) {
            return;
        }

        // 获取tenantId
        String firstLetter = PinyinUtil.getFirstLetter(corpName, "");
        String tenantId = "dd-" + firstLetter;

        List<TenantInfo> list = tenantInfoService.list(q -> q
                .likeRight(TenantInfo::getTenantId, "dd-" + firstLetter));

        List<String> tenantIdList = list.stream().map(TenantInfo::getTenantId).collect(Collectors.toList());

        if (list.size() > 0) {

            String tempTenantId;
            do {
                tempTenantId = tenantId + "-" + RandomUtil.randomString(3);
            } while (!tenantIdList.contains(tempTenantId));

            tenantId = tempTenantId;

        }

        // 创建租户
        TenantInfo tenantInfo = new TenantInfo();
        tenantInfo.setTenantId(tenantId);
        tenantInfo.setEnterpriseName(corpName);
        tenantInfo.setStatus(StatusConstant.ENABLE);
        tenantInfo.setFlowStatus(2);
        tenantInfo.setCorpid(corpid);

        // 创建部门
        ArrayList<SysDept> deptList = new ArrayList<>();
        addDeptList(corpid, deptList, tenantId, 1L, 0L);

        // 创建用户
        SysUser sysUser = new SysUser();
        // sysUser.setDeptId(sysDept.getDeptId());
        sysUser.setTenantId(tenantId);
        sysUser.setUserName("admin");
        sysUser.setNickName(corpName + "管理员");
        sysUser.setPassword(SecurityUtils.encryptPassword(RandomUtil.randomString(6)));
        sysUser.setStatus("0");
        sysUser.setCreateBy(SecurityUtils.getUsername());
        sysUser.setCreateTime(new Date());
        sysUser.setDdUserId(bizData.getJSONObject("auth_user_info").getString("userId"));
        sysUserService.save(sysUser);

        log.info("钉钉回调创建租户，tenantInfo：{}", JSON.toJSONString(tenantInfo));
        log.info("钉钉回调创建部门，deptList：{}", JSON.toJSONString(deptList));
        log.info("钉钉回调创建用户，sysUser：{}", JSON.toJSONString(sysUser));

        TransactionStatus transactionStatus = platformTransactionManager.getTransaction(transactionDefinition);
        try {

            // 保存租户
            tenantInfoService.save(tenantInfo);
            // 保存部门
            sysDeptService.saveBatch(deptList);
            // 提交事务
            platformTransactionManager.commit(transactionStatus);

            DynamicDataSourceContextHolder.poll();
        } catch (Exception e) {
            // 回滚事务
            platformTransactionManager.rollback(transactionStatus);
            throw new ServiceException("保存数据失败");
        }
    }

    /**
     * 添加部门列表
     */
    private void addDeptList(String corpid, List<SysDept> deptList, String tenantId, Long dingParentId, Long myParentId) {

        List<OapiV2DepartmentListsubResponse.DeptBaseResponse> tempDeptList = DingUtil.getDeptList(corpid, dingParentId);

        if (ObjectUtil.isEmpty(tempDeptList)) {
            return;
        }

        for (OapiV2DepartmentListsubResponse.DeptBaseResponse deptBaseResponse : tempDeptList) {
            SysDept sysDept = new SysDept();
            sysDept.setDeptId(IdWorker.getId());
            sysDept.setParentId(myParentId);
            sysDept.setDeptName(deptBaseResponse.getName());
            sysDept.setOrderNum(1);
            sysDept.setStatus("0");
            sysDept.setType(0);
            sysDept.setTenantId(tenantId);
            sysDept.setCreateTime(new Date());
            sysDept.setDingDeptId(deptBaseResponse.getDeptId());

            deptList.add(sysDept);

            addDeptList(corpid, deptList, tenantId, deptBaseResponse.getDeptId(), sysDept.getDeptId());
        }
    }

}
